@@ -137,6 +137,7 @@ psv<-function(samp,tree,compute.var=TRUE){
 
   if(is(tree)[1]=="phylo")
   {
+    if(is.null(tree$edge.length)){tree<-compute.brlen(tree, 1)}  #If phylo has no given branch lengths
     tree<-prune.sample(samp,tree)
     # Make sure that the species line up
     samp<-samp[,tree$tip.label]
@@ -262,8 +263,11 @@ pse<-function(samp,tree){
     flag=2
   }
 
+  samp<-as.matrix(samp)
+  
   if(is(tree)[1]=="phylo")
   {
+    if(is.null(tree$edge.length)){tree<-compute.brlen(tree, 1)}  #If phylo has no given branch lengths
     tree<-prune.sample(samp,tree) 
     # Make sure that the species line up
     samp<-samp[,tree$tip.label]
@@ -323,6 +327,7 @@ psc<-function(samp,tree){
 
   if(is(tree)[1]=="phylo")
   {
+    if(is.null(tree$edge.length)){tree<-compute.brlen(tree, 1)}  #If phylo has no given branch lengths
     tree<-prune.sample(samp,tree)
     # Make sure that the species line up
     samp<-samp[,tree$tip.label]
@@ -378,6 +383,7 @@ psv.spp<-function(samp,tree){
   }  
   if(is(tree)[1]=="phylo")
   {
+    if(is.null(tree$edge.length)){tree<-compute.brlen(tree, 1)}  #If phylo has no given branch lengths
     tree<-prune.sample(samp,tree)
     # Make sure that the species line up
     samp<-samp[,tree$tip.label]
@@ -400,7 +406,7 @@ psv.spp<-function(samp,tree){
   Cmatrix<-Cmatrix[indexcov,indexcov]
   samp<-samp[,indexcov]
 
-  obs.PSV<-mean(psv(samp,Cmatrix,compute.var=FALSE)[,1])
+  obs.PSV<-mean(psv(samp,Cmatrix,compute.var=FALSE)[,1],na.rm=TRUE)
 
   # numbers of locations and species
   nlocations<-dim(samp)[1]
@@ -411,7 +417,7 @@ psv.spp<-function(samp,tree){
   {
     spp.samp<-samp[,-j]
     spp.Cmatrix<-Cmatrix[-j,-j]
-    spp.PSV<-mean(psv(spp.samp,spp.Cmatrix,compute.var=FALSE)[,1])
+    spp.PSV<-mean(psv(spp.samp,spp.Cmatrix,compute.var=FALSE)[,1],na.rm=TRUE)
     spp.PSVs<-c(spp.PSVs,spp.PSV)
   }
   spp.PSVout<-(spp.PSVs-obs.PSV)/sum(abs(spp.PSVs-obs.PSV))
@@ -440,6 +446,7 @@ psd<-function(samp,tree,compute.var=TRUE){
 
 pd<-function(samp,tree){
   
+  if(is.null(tree$edge.length)){tree<-compute.brlen(tree, 1)}  #If phylo has no given branch lengths
   # Make sure that the species line up
   tree<-prune.sample(samp,tree)
   samp<-samp[,tree$tip.label]
